c
c     Description
c
c     Create an annular image mask, centered on the optical axis.
c
c     Authors
c
c     David Rupke
c
c     History
c
c     07oct02  DSNR  created
c

      implicit none

      integer NUMBERCHIPS
      parameter(NUMBERCHIPS=8)

      real xcen,ycen,xcenfp,ycenfp,rmin,rminsq,rmax,rmaxsq
      real x,y,xfp,yfp,rsq
      integer nchipcen,numchips,xbin,ybin,axlen(2)
      integer ichip,col,row,goodval,badval
      integer data(2048,4096)
      character*80 dummy,parfile,basename,suffix,imname,charin

      integer status,unit,blocksize,bitpix,naxis,ilen
      integer i,j,group,fpixel,nelements,ios
      logical simple,extend

      numchips = NUMBERCHIPS

c     Get optical axis location (chip and X and Y pixel locations).
 1    parfile = 'optaxis.dat'
      open (10,file=parfile,status="old",iostat=ios)
      if (ios .ne. 0) then 
         write(*,'(2(a))')
     $        "Error opening file containing optical axis: ",parfile
         stop
      end if
      read (10,*) dummy
      read (10,*) dummy
      read (10,*) nchipcen,xcen,ycen
      close(10)

c     Get minimum and maximum radii of annulus.
 10   write (*,11)
 11   format ("Enter minimum radius of unmasked annulus: [0] ",$)
      read (*,'(a)') charin
      if (charin(1:6) .eq. '      ') then 
         rmin = 0
      else
         read(charin,*) rmin
      end if
 12   write (*,13)
 13   format ("Enter maximum radius of unmasked annulus: [4000] ",$)
      read (*,'(a)') charin
      if (charin(1:6) .eq. '      ') then 
         rmax = 4000
      else
         read(charin,*) rmax
      end if
      rminsq = rmin*rmin
      rmaxsq = rmax*rmax

c     Enter good and bad values
 20   write (*,21)
 21   format ("Enter integer value for unmasked region: [1] ",$)
      read (*,'(a)') charin
      if (charin(1:6) .eq. '      ') then 
         goodval = 1
      else
         read(charin,*) goodval
      end if
 22   write (*,23)
 23   format ("Enter integer value for masked region: [0] ",$)
      read (*,'(a)') charin
      if (charin(1:6) .eq. '      ') then 
         badval=0
      else
         read(charin,*) badval
      end if

c     Compute chip center in focal plane coordinates
      call chiptofp(nchipcen,xcen,ycen,xcenfp,ycenfp)

c     Get reference image size, to determine binning.
c      x_max = imsize(1)
c      if (x_max .eq. 256) then
c         xbin = 8
c         ybin = 8
c         y_max = 512
c      else if (x_max .eq. 512) then
c         xbin = 4
c         ybin = 4
c         y_max = 1024
c      else if (x_max .eq. 1024) then
c         xbin = 2
c         ybin = 2
c         y_max = 2048
c      else if (x_max .eq. 2048) then
         xbin = 1               
         ybin = 1
         axlen(1) = 2048
         axlen(2) = 4096
c      else
c         write (*,'("Error in reference image: overscan not removed.")') 
c         go to 10
c      end if
      write(*,'("Using chip binning in X Y of ",i2,2x,i2)') xbin,ybin

c     Get output file base name
 30   write (*,31)
 31   format ("Enter base filename for output image: ",$)
      read (*,'(a)') basename

      status = 0

c Find end of basename. Image names with spaces will cause heartache!
      ilen = index(basename,' ')-1
c IMACS format: images are basenamec1.fits, etc.
c Only accomodates single-digit chip numbers (for now).

c     Cycle through chips and compute radius of each pixel in focal
c     plane coordinates.
      do ichip=1,numchips
C     Create image name.
C     '[compress PLIO]' option makes the mask a compressed image extension
c         write(suffix,'(a1,i1,a)') 'c',ichip,'.fits'
         write(suffix,'(a1,i1,a)') 'c',ichip,'.fits[compress PLIO]'
         imname = basename(1:ilen) // suffix
C     Delete the file if it already exists, so we can then recreate it.
         call deletefile(imname,status)
         if (status .gt. 0)call printerror(status)
C     Get a Logical Unit Number.
         call ftgiou(unit,status)
         if (status .gt. 0)call printerror(status)
C     Create the new file.
         blocksize=1
         call ftinit(unit,imname,blocksize,status)
         if (status .gt. 0)call printerror(status)

C     Initialize FITS parameters.
C     The EXTEND = TRUE parameter indicates that the FITS file
C     may contain extensions following the primary array.
C     BITPIX (16 vs. 8) makes no difference in the size of the output file.
         simple=.true.
         bitpix=16
         naxis=2
         extend=.true.
C     Write the required header keywords to the file.
         call ftphpr(unit,simple,bitpix,naxis,axlen,0,1,extend,status)
         if (status .gt. 0)call printerror(status)

         do col=1,axlen(1)
            do row=1,axlen(2)
               x = xbin*(col-0.5)+0.5
               y = ybin*(row-0.5)+0.5
               call chiptofp(ichip,x,y,xfp,yfp)
               rsq = (xcenfp-xfp)**2 + (ycenfp-yfp)**2
               if (rsq .gt. rmaxsq .or. rsq .le. rminsq) then
                  data(col,row) = badval
               else
                  data(col,row) = goodval
               end if
            end do
         end do
C     Write the array to the FITS file.
C     The last letter of the subroutine name defines the datatype of the
C     array argument; in this case the 'J' indicates that the array has an
C     integer*4 datatype. ('I' = I*2, 'E' = Real*4, 'D' = Real*8).
C     The 2D array is treated as a single 1-D array with NAXIS1 * NAXIS2
C     total number of pixels.  GROUP is seldom used parameter that should
C     almost always be set = 1.
C     Note that use of 'ftppri' yields junk here.
         group=1
         fpixel=1
         nelements=axlen(1)*axlen(2)
         call ftpprj(unit,group,fpixel,nelements,data,status)
         if (status .gt. 0)call printerror(status)
C     The FITS file must always be closed before exiting the program. 
C     Any unit numbers allocated with FTGIOU must be freed with FTFIOU.
         call ftclos(unit, status)
         call ftfiou(unit, status)
         if (status .gt. 0)call printerror(status)

      end do
      
      end
